<?php

/**
 * Class Kiranime_Ajax
 *
 * Handles AJAX requests for the Kiranime Theme.
 *
 * @package Kiranime
 * @since 3.5.0
 */
class Kiranime_Ajax {
	/**
	 * Constructor for Kiranime_Ajax class.
	 */
	public function __construct() {
		add_action( 'wp_ajax_get_nonce', [ $this, 'kiranime_ajax_get_nonce' ] );
		add_action( 'wp_ajax_nopriv_get_nonce', [ $this, 'kiranime_ajax_get_nonce' ] );
		add_action( 'wp_ajax_kiranime_dismiss_update_notification', [ $this, 'kiranime_ajax_kiranime_dismiss_update_notification' ] );
		add_action( 'wp_ajax_get_reports', [ $this, 'kiranime_ajax_get_reports' ] );
		add_action( 'wp_ajax_nopriv_register_user', [ $this, 'register_user' ] );
		add_action( 'wp_ajax_nopriv_login_user', [ $this, 'login_user' ] );
		add_action( 'wp_ajax_recovery_user', [ $this, 'recovery_user' ] );
		add_action( 'wp_ajax_nopriv_recovery_user', [ $this, 'recovery_user' ] );
		add_action( 'wp_ajax_reset_user', [ $this, 'reset_user' ] );
		add_action( 'wp_ajax_nopriv_reset_user', [ $this, 'reset_user' ] );
	}

	public function register_user() {
		check_ajax_referer( 'ajax-register-nonce', 'register_nonce' );

		$username         = sanitize_text_field( wp_unslash( $_POST['username'] ?? '' ) );
		$password         = $_POST['password'] ?? '';
		$confirm_password = $_POST['confirm_password'] ?? '';
		$email            = sanitize_email( $_POST['email'] ?? '' );
		$token            = sanitize_text_field( wp_unslash( $_POST['cf-turnstile-response'] ?? '' ) );

		$errors = [];

		if ( empty( $username ) ) {
			$errors[] = __( 'Username field is required.', 'kiranime' );
		} elseif ( ! validate_username( $username ) ) {
			$errors[] = __( 'Invalid username. Only lowercase letters, numbers, and underscores are allowed.', 'kiranime' );
		} elseif ( username_exists( $username ) ) {
			$errors[] = __( 'Username already exists.', 'kiranime' );
		}

		if ( ! empty( $password ) && $password !== $confirm_password ) {
			$errors[] = __( 'Passwords do not match.', 'kiranime' );
		}

		if ( get_option( '__use_captcha', false ) ) {
			$c = new Kiranime_Captcha();
			if ( ! $c->validate( $token ) ) {
				wp_send_json_error( __( 'Invalid captcha token.', 'kiranime' ), 400 );
			}
		}

		if ( ! empty( $errors ) ) {
			wp_send_json_error( $errors, 400 );
		}

		$result = Kira_User::register( $email, $username, $password );

		if ( empty( $result['data']['status'] ) ) {
			wp_send_json_error( $result['data'], $result['status'] );
		}

		wp_send_json_success( $result['data'], $result['status'] );
	}

	public function login_user() {
		check_ajax_referer( 'ajax-login-nonce', 'login_nonce' );

		$username    = sanitize_text_field( wp_unslash( $_POST['username'] ?? '' ) );
		$password    = $_POST['password'] ?? '';
		$token       = sanitize_text_field( wp_unslash( $_POST['cf-turnstile-response'] ?? '' ) );
		$remember_me = isset( $_POST['remember_me'] ) && 'on' === sanitize_text_field( wp_unslash( $_POST['remember_me'] ) );

		if ( get_option( '__use_captcha', false ) ) {
			$c = new Kiranime_Captcha();
			if ( ! $c->validate( $token ) ) {
				wp_send_json_error( __( 'Invalid captcha token.', 'kiranime' ), 400 );
			}
		}
		$login = Kira_User::login( $username, $password, $remember_me );

		if ( empty( $login['user'] ) ) {
			wp_send_json_error( $login['data'], $login['status'] );
		}

		wp_set_auth_cookie( $login['data']['user']->ID, $remember_me, is_ssl() );
		wp_set_current_user( $login['data']['user']->ID );
		wp_send_json_success( $login['data'] );
	}

	public function recovery_user() {
		check_ajax_referer( 'ajax-recovery-nonce', 'recovery_nonce' );

		$login = sanitize_text_field( wp_unslash( $_POST['userlogin'] ?? '' ) );
		$token = sanitize_text_field( wp_unslash( $_POST['cf-turnstile-response'] ?? '' ) );

		if ( get_option( '__use_captcha', false ) ) {
			$c = new Kiranime_Captcha();
			if ( ! $c->validate( $token ) ) {
				wp_send_json_error( __( 'Invalid captcha token.', 'kiranime' ), 400 );
			}
		}

		$recovery = ( new Kira_User() )->get_recovery_verification_code( $login );

		if ( empty( $recovery['data']['status'] ) ) {
			wp_send_json_error( $recovery['data'], $recovery['status'] );
		}

		wp_send_json_success( $recovery['data'] );
	}

	public function reset_user() {
		check_ajax_referer( 'ajax-reset-nonce', 'reset_nonce' );

		$verification_token = sanitize_text_field( wp_unslash( $_POST['verification_token'] ?? '' ) );
		$userlogin          = sanitize_text_field( wp_unslash( $_POST['userlogin'] ?? '' ) );
		$password           = sanitize_text_field( wp_unslash( $_POST['password'] ?? '' ) );
		$confirm_password   = sanitize_text_field( wp_unslash( $_POST['confirm_password'] ?? '' ) );
		$token              = sanitize_text_field( wp_unslash( $_POST['cf-turnstile-response'] ?? '' ) );

		if ( get_option( '__use_captcha', false ) ) {
			$c = new Kiranime_Captcha();
			if ( ! $c->validate( $token ) ) {
				wp_send_json_error( __( 'Invalid captcha token.', 'kiranime' ), 400 );
			}
		}

		$user = ( new Kira_User() )->reset_my_password(
			[
				'userlogin'          => $userlogin,
				'password'           => $password,
				'confirm_password'   => $confirm_password,
				'verification_token' => $verification_token,
			]
		);

		if ( empty( $user['data']['status'] ) ) {
			wp_send_json_error( $user['data'], $user['status'] );
		}

		wp_send_json_success( $user['data'] );
	}

	public function kiranime_ajax_get_reports() {
		$page = absint( $_GET['page'] ?? 1 );

		$reports     = new WP_Query(
			[
				'post_type'      => 'reported',
				'posts_per_page' => 20,
				'order'          => 'desc',
				'orderby'        => 'date',
				'paged'          => $page,
			]
		);
		$report_data = [];
		foreach ( $reports->posts as $post ) {
			$report_data[] = [
				'post'  => $post,
				'value' => get_post_meta( $post->ID, 'reported_value', true ),
				'link'  => get_post_meta( $post->ID, 'reported_episode', true ),
			];
		}

		wp_send_json_success(
			[
				'reports' => $report_data,
				'pages'   => $reports->max_num_pages,
				'total'   => $reports->found_posts,
			]
		);
		wp_die();
	}

	/**
	 * Retrieves a nonce for AJAX requests.
	 *
	 * This function is used to create a nonce for AJAX requests to prevent unauthorized access.
	 * It creates a nonce using the 'wp_rest' action and sends it back as a JSON response.
	 *
	 * @return mixed Returns a JSON response with the nonce.
	 */
	public function kiranime_ajax_get_nonce() {
		$nonce = wp_create_nonce( 'wp_rest' );
		return wp_send_json_success( [ 'nonce' => $nonce ] );
	}
	/**
	 * Dismisses the update notification for the Kiranime theme.
	 *
	 * This function deletes the 'kiranime_update_available' option, effectively dismissing the update notification.
	 * It then sends a JSON response indicating the success of the operation.
	 *
	 * @return mixed Returns a JSON response with the status of the operation.
	 *               - 'status': The result of the delete_option() function.
	 *                           If the option was successfully deleted, this will be true.
	 *                           If the option did not exist, this will be false.
	 */
	public function kiranime_ajax_kiranime_dismiss_update_notification() {
		$options = delete_option( 'kiranime_update_available' );
		return wp_send_json_success( [ 'status' => $options ] );
	}
}
