<?php

class Kiranime_Captcha {
	private $site_key   = '';
	private $secret_key = '';
	private $response   = '';

	/**
	 * Constructs a new Kiranime_Captcha instance.
	 *
	 * Initializes the site key and secret key for Cloudflare Turnstile captcha.
	 * The site key and secret key are fetched from the WordPress options.
	 * If the options are not set, empty strings are assigned to the site key and secret key.
	 */
	public function __construct() {
		$this->site_key   = get_option( 'turnstile_site_key', '' );
		$this->secret_key = get_option( 'turnstile_secret_key', '' );
	}

	/**
	 * Validates the response from Cloudflare Turnstile.
	 *
	 * @param string $response The response from the Turnstile widget.
	 *
	 * @return bool True if the response is valid, false otherwise.
	 */
	public function validate( $response ) {
		if ( empty( $response ) ) {
			return false;
		}

		$this->response = $response;

		$data = [
			'secret'   => $this->secret_key,
			'response' => $this->response,
			'remoteip' => $this->get_client_ip(),
		];

		$req = wp_remote_post(
			'https://challenges.cloudflare.com/turnstile/v0/siteverify',
			[
				'body'      => $data,
				'sslverify' => false,
			]
		);

		if ( is_wp_error( $req ) ) {
			return false;
		}

		$result = Kira_Utility::safe_decode( wp_remote_retrieve_body( $req ), true );

		if ( empty( $result ) ) {
			return false;
		}

		return isset( $result['success'] ) && true === $result['success'];
	}

	/**
	 * Retrieves the client's IP address.
	 *
	 * This function checks the HTTP headers for the client's IP address. It first checks for the
	 * 'HTTP_CLIENT_IP' header, then 'HTTP_X_FORWARDED_FOR', and finally 'REMOTE_ADDR'.
	 *
	 * @return string The client's IP address.
	 */
	private function get_client_ip() {
		$ip = '';

		if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
			$ip = sanitize_text_field( wp_unslash( $_SERVER['HTTP_CLIENT_IP'] ) );
		} elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
			$ip = sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) );
		} else {
			$ip = sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ?? '' ) );
		}

		return $ip;
	}

	/**
	 * Renders the Cloudflare Turnstile captcha.
	 *
	 * This function checks if the site key is set and then renders the Turnstile captcha
	 * using the provided site key. If the site key is not set, it returns an empty string.
	 *
	 * @return string The rendered Turnstile captcha HTML or an empty string if the site key is not set.
	 */
	public function render_captcha( $template = '' ) {
		if ( empty( $this->site_key ) ) {
			return '';
		}
		?>
		<div style="display: flex; justify-content: center; margin-block: 1rem; flex-flow: row;" id="<?php echo esc_attr( $template ); ?>" data-sitekey="<?php echo esc_attr( $this->site_key ); ?>">
			
		</div>
		<?php
	}
}
